from typing import List
from starlette.requests import Request
#import ray
from ray import serve

@serve.deployment(num_replicas=1, ray_actor_options={"num_cpus": 3, "num_gpus": 0})
class BatchTextGenerator:
    def __init__(self, pipeline_key: str, model_key: str):
        from transformers import pipeline
        self.model = pipeline(pipeline_key, model_key)

    @serve.batch(batch_wait_timeout_s=10,max_batch_size=4)
    async def handle_batch(self, inputs: List[str]) -> List[str]:
        print("Our input array has length:", len(inputs))

        results = self.model(inputs)
        return [result[0]["generated_text"] for result in results]

    async def __call__(self, request: Request) -> List[str]:
        return await self.handle_batch(request.query_params["text"])

generator = BatchTextGenerator.options(route_prefix='/').bind("text-generation", "gpt2")

#ray.init(address='auto')
#serve.start(http_options={'host':'0.0.0.0'})
#serve.run(generator)
